import tensorflow as tf
import keras as ks
import numpy as np
import sys
import math
from keras.layers  import Dense, Conv2D,BatchNormalization,Activation,MaxPooling2D
from keras.layers import Input,Dropout,MaxPooling1D,concatenate,GlobalAveragePooling2D,Lambda,multiply
from keras.models import Model
from keras.optimizers import RMSprop, Adam,SGD
import os
from keras import backend as K
from keras import regularizers

EPS=0.00000000001
os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
os.environ["CUDA_VISIBLE_DEVICES"] = "0" # Select CUDA device to run on.
SHAPE=(2500,12,1) #Shape of the input. ECG model takes 2500 points sampled at 4 micro-seconds/point for 12 lead.
batch_size=22 #Define sample numbers per batch.

testData=np.load('Input.npy') #Load input data. Input data should be compiled as numpy arrays with (sample numbers, time, lead, 1)
dlen=testData.shape[0]
testData=testData.reshape((dlen,)+SHAPE) #Reshaping to make sure the shape is correct.

def multi_conv2D(x,num_kernel,activation="relu"): #Multi-2D-CNN module.
    kreg = None
    a=Conv2D(int(num_kernel/2),1,activation=activation,padding="same",kernel_regularizer=kreg)(x)
    a=BatchNormalization()(a)
    a=Conv2D(num_kernel,3,activation=activation,padding="same",kernel_regularizer=kreg)(a)
    a=BatchNormalization()(a)
    b=Conv2D(int(num_kernel/3),1,activation=activation,padding="same",kernel_regularizer=kreg)(x)
    b = BatchNormalization()(b)
    b=Conv2D(int(num_kernel/2),3,activation=activation,padding="same",kernel_regularizer=kreg)(b)
    b = BatchNormalization()(b)
    b = Conv2D(num_kernel, 3, activation=activation, padding="same",kernel_regularizer=kreg)(b)
    b = BatchNormalization()(b)
    c=Conv2D(int(num_kernel/3),1,activation=activation,padding="same",kernel_regularizer=kreg)(x)
    c = BatchNormalization()(c)
    res=concatenate([a,b,c],axis=-1)
    res=BatchNormalization()(res)
    return res

def get_models_2D(): #Model structure
    input1 = Input(SHAPE)
    initial_kernel_num = 64
    x = input1
    x = Conv2D(initial_kernel_num, (7,3),strides=(2,1), activation="relu", padding="same")(x)
    x = BatchNormalization()(x)
    x = multi_conv2D(x, initial_kernel_num)
    x = multi_conv2D(x, initial_kernel_num)
    x = MaxPooling2D(pool_size=(3,1))(x)
    x = multi_conv2D(x, int(initial_kernel_num*1.5))
    x = multi_conv2D(x, int(initial_kernel_num*1.5))
    x = MaxPooling2D(pool_size=(3,1))(x)
    x = multi_conv2D(x, int(initial_kernel_num*2))
    x = multi_conv2D(x, int(initial_kernel_num*2))
    x = MaxPooling2D(pool_size=(2,1))(x)
    x = multi_conv2D(x, initial_kernel_num*3)
    x = multi_conv2D(x, initial_kernel_num*3)
    x = multi_conv2D(x, initial_kernel_num*4)
    x = MaxPooling2D(pool_size=(2,1))(x)
    x = multi_conv2D(x, initial_kernel_num*5)
    x = multi_conv2D(x, initial_kernel_num*6)
    x = multi_conv2D(x, initial_kernel_num*7)
    x = MaxPooling2D(pool_size=(2,1))(x)
    x = multi_conv2D(x, initial_kernel_num*8)
    x = multi_conv2D(x, initial_kernel_num*8)
    x = multi_conv2D(x, initial_kernel_num*8)
    x =MaxPooling2D(pool_size=(2,1))(x)
    x = multi_conv2D(x, initial_kernel_num*12)
    x = multi_conv2D(x, initial_kernel_num*14)
    x = multi_conv2D(x, initial_kernel_num*16)
    x = GlobalAveragePooling2D()(x)
    a1 = Dropout(0.5)(x)
    a1 = Dense(1)(a1)
    b1 = Activation('sigmoid')(a1)
    model = Model(inputs=input1, outputs=b1)
    return model

model = get_models_2D()
model.summary()

print('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa')

model.load_weights(os.path.join("./n",'model_weights.hdf5')) #Read in model weight.

predictions = model.predict(testData, batch_size=batch_size) #Run model on the input data and get predictions
print(predictions) #Print predictions
