import tensorflow as tf
import keras as ks
import numpy as np
import sys
import math
import random
import pandas as pd
from keras.layers  import Dense,Conv3D,BatchNormalization,Activation,MaxPooling3D,
from keras.layers import Input,Dropout,concatenate,AveragePooling3D,GlobalAveragePooling3D,Lambda,multiply
from keras.models import Model
from keras.layers.core import Dense, Activation
from numpy.random import seed
from keras.optimizers import RMSprop, Adam,SGD
import os
from keras import backend as K
from keras import regularizers
import lz4.frame
EPS=0.00000000001
os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
os.environ["CUDA_VISIBLE_DEVICES"] = "0" # Select CUDA device to run on.
SHAPE=(30,299,299,1) #Shape of the input. 30 frames of 30 fps videos of 299x299.
batch_size=6 #Define sample numbers per batch.
testData=np.load('Input.npy') #Load input data. Input data should be compiled as numpy arrays with (sample numbers, time, y, x, 1)
testScale=np.load('InputScale.npy') # Load scale information, which is a single float number for video representing cm/pixel

kernel_init = ks.initializers.glorot_uniform()
bias_init = ks.initializers.Constant(value=0.2)

testData=[testData,testScale,1/(testScale+EPS)] #Compile video with scale to contruct the input.


def multi_conv3D(x,filters_1x1,filters_3x3_reduce,filters_3x3,filters_5x5_reduce,filters_5x5,filters_pool_proj,trainable=True):
    conv_1x1 = Conv3D(filters_1x1, (1,1,1), padding='same', activation='relu',kernel_initializer=kernel_init, bias_initializer=bias_init,trainable=trainable)(x)
    conv_1x1 = BatchNormalization(scale=False,trainable=trainable)(conv_1x1)
    conv_3x3 = Conv3D(filters_3x3_reduce, (1,1,1), padding='same', activation='relu',kernel_initializer=kernel_init, bias_initializer=bias_init,trainable=trainable)(x)    
    conv_3x3 = BatchNormalization(scale=False,trainable=trainable)(conv_3x3)
    conv_3x3 = Conv3D(filters_3x3, (3,3,3), padding='same', activation='relu',kernel_initializer=kernel_init, bias_initializer=bias_init,trainable=trainable)(conv_3x3)
    conv_3x3 = BatchNormalization(scale=False,trainable=trainable)(conv_3x3)
    conv_5x5 = Conv3D(filters_5x5_reduce, (1,1,1), padding='same', activation='relu',kernel_initializer=kernel_init, bias_initializer=bias_init,trainable=trainable)(x)
    conv_5x5 = BatchNormalization(scale=False,trainable=trainable)(conv_5x5)
    conv_5x5 = Conv3D(filters_5x5, (3,3,3),strides=(1,1,1), padding='same', activation='relu',kernel_initializer=kernel_init, bias_initializer=bias_init,trainable=trainable)(conv_5x5)
    conv_5x5 = BatchNormalization(scale=False,trainable=trainable)(conv_5x5)
    conv_5x5 = Conv3D(filters_5x5, (3,3,3),strides=(1,1,1), padding='same', activation='relu',kernel_initializer=kernel_init, bias_initializer=bias_init,trainable=trainable)(conv_5x5)
    conv_5x5 = BatchNormalization()(conv_5x5)
    pool_proj = MaxPooling3D((3,3,3), strides=(1,1,1), padding='same')(x)
    pool_proj = Conv3D(filters_pool_proj, (1,1,1), padding='same', activation='relu',kernel_initializer=kernel_init, bias_initializer=bias_init,trainable=trainable)(pool_proj)
    output = concatenate([conv_1x1, conv_3x3, conv_5x5, pool_proj], axis=4)
    return output


def get_models(): #Model structure
    trainable = True
    inputEco = Input(shape=SHAPE)
    inputScale = Input(shape=(1,))
    inputInvScale = Input(shape=(1,))
    filters = 64
    x = inputEco
    x = Conv3D(filters, (7, 7, 7), padding='same', strides=(2, 2, 2), activation='relu', kernel_initializer=kernel_init,bias_initializer=bias_init, trainable=trainable)(x)
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    x = MaxPooling3D(pool_size=(3, 3, 3), padding='same', strides=(2, 2, 2))(x)
    x = Conv3D(filters, (1, 1, 1), padding='same', strides=(1, 1, 1), activation='relu', kernel_initializer=kernel_init,
              bias_initializer=bias_init, trainable=trainable)(x)
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    x = Conv3D(filters * 3, (3, 3, 3), padding='same', strides=(1, 1, 1), activation='relu',
              kernel_initializer=kernel_init, bias_initializer=bias_init, trainable=trainable)(x)
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    x = multi_conv3D(x, filters_1x1=filters, filters_3x3_reduce=int(filters * 1.5), filters_3x3=filters * 4,
                        filters_5x5_reduce=int(filters / 4), filters_5x5=int(filters / 2),
                        filters_pool_proj=int(filters / 2), trainable=trainable)
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    x = multi_conv3D(x, filters_1x1=filters * 2, filters_3x3_reduce=filters * 2, filters_3x3=filters * 3,
                        filters_5x5_reduce=int(filters / 2), filters_5x5=filters * 3, filters_pool_proj=filters,
                        trainable=trainable)
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    x = MaxPooling3D(pool_size=(1, 3, 3), padding='same', strides=(2, 2, 2))(x)  # (1,3,3)


    x = multi_conv3D(x, filters_1x1=filters * 3, filters_3x3_reduce=int(filters * 1.5),
                        filters_3x3=int(filters * 3.25), filters_5x5_reduce=int(filters / 4),
                        filters_5x5=int(filters * 0.75), filters_pool_proj=filters, trainable=trainable)
    x = BatchNormalization(scale=False, trainable=trainable)(x)

    x = multi_conv3D(x, filters_1x1=int(filters * 2.5), filters_3x3_reduce=int(filters * 1.75),
                        filters_3x3=int(filters * 3.5), filters_5x5_reduce=int(filters * 0.375), filters_5x5=filters,
                        filters_pool_proj=filters, trainable=trainable)
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    x = multi_conv3D(x, filters_1x1=filters * 2, filters_3x3_reduce=filters * 2, filters_3x3=filters * 4,
                        filters_5x5_reduce=int(filters * 0.375), filters_5x5=filters, filters_pool_proj=filters,
                        trainable=trainable)
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    x = multi_conv3D(x, filters_1x1=int(filters * 1.75), filters_3x3_reduce=int(filters * 2.25),
                        filters_3x3=int(filters * 4.5), filters_5x5_reduce=int(filters / 2), filters_5x5=filters,
                        filters_pool_proj=filters, trainable=trainable)
    x = BatchNormalization(scale=False, trainable=trainable)(x)

    x = multi_conv3D(x, filters_1x1=filters * 4, filters_3x3_reduce=int(filters * 2.5), filters_3x3=filters * 5,
                        filters_5x5_reduce=int(filters / 2), filters_5x5=filters * 2, filters_pool_proj=filters * 2,
                        trainable=trainable)
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    x = MaxPooling3D((1, 3, 3), strides=(2, 2, 2))(x)


    x = multi_conv3D(x, filters_1x1=filters * 4, filters_3x3_reduce=int(filters * 2.5), filters_3x3=filters * 5,
                        filters_5x5_reduce=int(filters / 2), filters_5x5=filters * 2, filters_pool_proj=filters * 2,
                        trainable=trainable)
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    x = multi_conv3D(x, filters_1x1=filters * 6, filters_3x3_reduce=filters * 3, filters_3x3=filters * 6,
                        filters_5x5_reduce=int(filters * 0.75), filters_5x5=filters * 2, filters_pool_proj=filters * 2,
                        trainable=trainable)
    x = BatchNormalization(scale=False, trainable=trainable)(x)

    x = MaxPooling3D(pool_size=(1, 2, 2), trainable=trainable)(x)

    x = multi_conv3D(x, filters_1x1=filters * 6, filters_3x3_reduce=filters * 3, filters_3x3=filters * 6,
                        filters_5x5_reduce=int(filters * 0.75), filters_5x5=filters * 2, filters_pool_proj=filters * 2,
                        trainable=trainable)
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    x = multi_conv3D(x, filters_1x1=filters * 6, filters_3x3_reduce=filters * 3, filters_3x3=filters * 6,
                        filters_5x5_reduce=int(filters * 0.75), filters_5x5=filters * 2, filters_pool_proj=filters * 2,
                        trainable=trainable)
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    x = multi_conv3D(x, filters_1x1=filters * 8, filters_3x3_reduce=filters * 4, filters_3x3=filters * 8,
                        filters_5x5_reduce=int(filters * 1), filters_5x5=filters * 3, filters_pool_proj=filters * 3,
                        trainable=trainable)
    x = BatchNormalization(scale=False, trainable=trainable)(x)

    g = GlobalAveragePooling3D()(x)
    x = Dropout(0.4)(g)
    x = Dense(100, activation="relu", trainable=trainable)(x)
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    a = inputScale
    a = Dense(100,activation='relu')(a)
    a = BatchNormalization(scale=False, trainable=trainable)(a)
    a = multiply([x, a])
    b = inputInvScale
    b=Dense(100,activation='relu')(b)
    b = BatchNormalization(scale=False, trainable=trainable)(b)
    b = multiply([x, b])
    x = concatenate([x, a, b])
    x = BatchNormalization(scale=False, trainable=trainable)(x)
    xa = Dense(1,activation='sigmoid', name="reg")(x)

    model = Model([inputEco, inputScale, inputInvScale], xa)
    return model



model = get_models()
model.summary()
print('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa')
model.load_weights(os.path.join("./n",'LSTM_CNN_BEST_model.hdf5')) #Read in model weight.
predictions = model.predict(testData, batch_size=batch_size) #Run model on the input data and get predictions

print(predictions)#Print predictions
